VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Convert"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Created by:  Kristian S. Stangeland
' Homepage:    http://www.stangeland.tk/
' Date:        29.08.2003
' Updated:     17.05.2005

'Copyright (C) 2005 Kristian S. Stangeland

'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' API-calls used primarily to access/manipulate strings faster
Private Declare Function VarPtrArray Lib "msvbvm50.dll" Alias "VarPtr" (Ptr() As Any) As Long
Private Declare Sub ZeroMemory Lib "kernel32" Alias "RtlZeroMemory" (pDst As Any, ByVal ByteLen As Long)
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (pDst As Any, pSrc As Any, ByVal ByteLen As Long)

' Used in conjunction with the below
Private Type SAFEARRAYBOUND
    cElements As Long
    lLbound As Long
End Type

' The safe-array structure used in the fast string access
Private Type SAFEARRAY1D
    cDims As Integer
    fFeatures As Integer
    cbElements As Long
    cLocks As Long
    pvData As Long
    Bounds(0) As SAFEARRAYBOUND
End Type

' Different constant in string manipulation
Private Const QuotationMark As Integer = 34
Private Const Apostrophe As Integer = 39
Private Const LessThan As Integer = 60
Private Const GreaterThan As Integer = 62
Private Const SpaceChar As Integer = 32
Private Const Slash As Integer = 47
Private Const MinusSign As Integer = 45
Private Const EqualSign As Integer = 61
Private Const QuestionMark As Integer = 63
Private Const ExclamationMark As Integer = 33
Private Const Ampersand As Integer = 38
Private Const NumberSign As Integer = 35
Private Const Semicolon As Integer = 59
Private Const LeftParant As Integer = 40
Private Const RightParant As Integer = 41
Private Const FullStop As Integer = 46
Private Const NoBreakSpace As Integer = 160

Private Type KeyWord
    Navn As String
    Only As Variant
End Type

Private KeyWords() As KeyWord
Private KeyCount As Long

' The StringOutput-variable
Public StringInput As StringBuilder
Public StringOutput As StringBuilder

' Different properties relatated to the making of the result
Public CodeBlue As String
Public CodeGreen As String
Public CodeLine As String
Public CodeEndTag As String
Public CodeEndDoc As String
Public LineBreak As String
Public CodeBegin As StringBuilder
Public FileName As String
Public MakeDocument As Boolean
Public TabReplacement As String
Public Seperator As Boolean
Public Font As New StdFont

Public Function GetFolderList(Path As String, ByVal Extension As String, Optional Attributers As VbFileAttribute = vbNormal) As Collection

    Dim sFile As String, AllowAll As Boolean
    
    ' Create new collection
    Set GetFolderList = New Collection
    
    ' Set variables
    sFile = Dir(ValidPath(Path), Attributers)
    
    ' Search through directory
    Do Until sFile = ""
        
        ' Only proceed if, we by the internal regular expressions, find the file to follow a specific pattern
        If sFile Like Extension Then
            GetFolderList.Add sFile
        End If
    
        ' Find the next file in the folder
        sFile = Dir
    Loop

End Function

Public Function ReadFile(sFile As String) As String

    Dim Free As Long

    ' Get a free handle
    Free = FreeFile
    
    ' Open the visual basic file
    Open sFile For Binary As #Free
        
        ' Allocate a buffer
        ReadFile = Space(LOF(Free))
        
        ' Get all the data
        Get #Free, , ReadFile
        
    Close #Free

End Function

Public Sub LoadFile(sFile As String)

    ' Clear the input
    Set StringInput = New StringBuilder

    ' Set the buffer
    StringInput.Append ReadFile(sFile)

End Sub

Public Function Convert() As Long
    
    Dim strExt As String, objTemp As New StringBuilder
        
    ' Create the StringOutput
    Set StringOutput = New StringBuilder
      
    ' Add the begin-string
    StringOutput.Append CodeBegin.ToString
      
    ' Convert to valid HTML
    ValidHTML StringInput.ToString, objTemp
    
    ' Then convert text
    If LineBreak <> "" Then
        StringOutput.Append Replace(ConvertText(objTemp.ToString), vbCrLf, LineBreak & vbCrLf)
    Else
        StringOutput.Append ConvertText(objTemp.ToString)
    End If
    
    ' Add the end-string
    StringOutput.Append CodeEndDoc
    
End Function

Public Sub AddKeyWord(ByVal Code As String, ParamArray Only() As Variant)

    ReDim Preserve KeyWords(KeyCount)
    
    KeyWords(KeyCount).Navn = LCase(Code)
    
    If UBound(Only) >= 0 Then
        KeyWords(KeyCount).Only = Only
    End If
    
    KeyCount = KeyCount + 1

End Sub

Public Function GetFileName(Text, Optional sMatch As String = "\") As String
    
    GetFileName = Right(Text, Len(Text) - InStrRev(Text, sMatch))

End Function

Private Function ConvertText(sText As String) As String

    Dim Lines As Variant, destWords As Variant, resultWords As Variant, Tell As Long
    Dim Key As Long, Comment As Boolean, bMake As Boolean, OnlyLoop As Long, currChar As Integer
    Dim Temp As Long, Word(4) As Boolean, sSearch As String, bVerified As Boolean
    
    ' Split up in lines
    Lines = Split(sText, vbCrLf)

    ' Go through all lines
    For Tell = LBound(Lines) To UBound(Lines)
    
        ' Only go further if this line isn't empty
        If LenB(Lines(Tell)) <> 0 Then
    
            ' Split up in words
            destWords = SplitWords(Lines(Tell))
    
            ' The array containing the result
            ReDim resultWords(LBound(destWords) To UBound(destWords))

            ' Go through all words
            For Temp = LBound(destWords) To UBound(destWords)
            
                ' Check for comments
                If (InStr(1, destWords(Temp), "'") <> 0) Or (InStr(1, destWords(Temp), "REM") <> 0) Then
                    destWords(Temp) = CodeGreen & destWords(Temp)
                    Comment = True
                End If
                
                ' Reset array
                For Key = LBound(Word) To UBound(Word)
                    Word(Key) = False
                Next
                
                ' Get the string to search for
                sSearch = LCase(Replace(destWords(Temp), ",", ""))
                
                ' Reset variable
                bMake = False

                ' Go through all key-words
                For Key = LBound(KeyWords) To UBound(KeyWords)
                
                    ' See if the first string is less or equal to the searching string
                    If LenB(KeyWords(Key).Navn) <= LenB(sSearch) Then
                
                        ' Set as normal
                        bVerified = True
                
                        ' The extra characters must be valid
                        If LenB(KeyWords(Key).Navn) <> LenB(sSearch) Then
                        
                            ' Get the last character according to the keyword's name
                            currChar = AscW(Mid(sSearch, Len(KeyWords(Key).Navn) + 1))
                         
                            ' Should we continue
                            bVerified = CBool(currChar < 33 Or currChar = LeftParant Or currChar = RightParant Or currChar = FullStop Or currChar = 32)
                        
                        End If

                        ' Is this keyword corresponding?
                        If KeyWords(Key).Navn = Left(sSearch, Len(KeyWords(Key).Navn)) And bVerified And (Not Comment) Then
                    
                            If KeyWords(Key).Navn = "function" Then Word(0) = True
                            If KeyWords(Key).Navn = "sub" Then Word(1) = True
                            If KeyWords(Key).Navn = "propety" Then Word(2) = True
                            If KeyWords(Key).Navn = "exit" Then Word(3) = True
                            If KeyWords(Key).Navn = "end" Then Word(4) = True
                            
                            If Temp > 1 And Not IsEmpty(KeyWords(Key).Only) Then
                            
                                For OnlyLoop = LBound(KeyWords(Key).Only) To UBound(KeyWords(Key).Only)
                                    If LCase(Left(destWords(Temp - 1), Len(destWords(Temp - 1)) - 1)) = LCase(KeyWords(Key).Only(OnlyLoop)) Then
                                    
                                        ' Make this section blue
                                        bMake = True
                                        
                                        
                                        ' We are finish here
                                        Exit For
                                        
                                    End If
                                Next
                            
                            Else
                                
                                bMake = True
                            
                            End If
                            
                            If bMake Then
                                
                                ' Make the section blue
                                resultWords(Temp) = CodeBlue & destWords(Temp) & CodeEndTag
                                
                                ' We are finish searching
                                Exit For
                                
                            End If
                    
                        End If
                    
                    End If
                    
                Next
                
                ' If we haven't make this section blue, add it as regular text
                If Not bMake Then
                
                    resultWords(Temp) = destWords(Temp)
                    
                End If
                
            Next
            
            ' All comments ends at the next line
            If Comment Then
                Comment = False
                resultWords(UBound(resultWords)) = destWords(UBound(resultWords)) & CodeEndTag
            End If
        
            ' Change the line
            If Seperator Then
                Lines(Tell) = IIf((Word(0) Or Word(1) Or Word(2)) And (Word(3) = False And Word(4) = False), CodeLine, "") & Join(resultWords, "")
            Else
                Lines(Tell) = Join(resultWords, "")
            End If

        End If
    
    Next
    
    ' Return code
    ConvertText = Join(Lines, vbCrLf)

End Function

Friend Function SplitWords(sText) As Variant
    
    Dim Tell As Long, lastPos As Long, lngCount As Long, currChar As Integer, aBuff()
    
    ' This is the last position
    lastPos = 1
    
    ' Go trough the entire string
    For Tell = 1 To Len(sText)
    
        ' Get the current char
        currChar = AscW(Mid(sText, Tell, 1))
    
        If currChar < 33 Or currChar = LeftParant Or currChar = RightParant Or currChar = FullStop Or currChar = NoBreakSpace Or Tell = Len(sText) Then
        
            ' Reallocate array
            ReDim Preserve aBuff(lngCount)
            
            ' Set array
            aBuff(lngCount) = Mid(sText, lastPos, Tell - lastPos + 1)
            
            ' Increse count
            lngCount = lngCount + 1
        
            ' Set the last position
            lastPos = Tell + 1
        
        End If
    
    Next
    
    ' Return array
    SplitWords = aBuff
    
End Function

Friend Sub ValidHTML(sText As String, clsString As StringBuilder)

    Dim sData As String, sResult As String, intChar() As Integer, currChar As Integer
    Dim SafeArray As SAFEARRAY1D, Tell As Long, lngLast As Long, prevChar As Integer
        
    ' Let the array point to the string
    With SafeArray
        .cDims = 1
        .Bounds(0).lLbound = 1
        .Bounds(0).cElements = Len(sText)
        .pvData = StrPtr(sText)
    End With
    
    ' Use copy memory for this purpose
    CopyMemory ByVal VarPtrArray(intChar), VarPtr(SafeArray), 4

    ' Let it begin at the first position
    lngLast = 1

    ' Go trough the entire array
    For Tell = 1 To UBound(intChar)
    
        ' Get the current char
        currChar = intChar(Tell)
            
        If (currChar = LessThan) Or (currChar = GreaterThan) Or (currChar = Ampersand) _
         Or (currChar = SpaceChar And prevChar = SpaceChar) Or (currChar = vbKeyTab) Or (currChar > 127) Then
    
            ' Add the text before
            clsString.Append Mid(sText, lngLast, Tell - lngLast)
            
            ' Add the special enitity
            Select Case currChar
                Case vbKeyTab
                    clsString.Append TabReplacement
                Case SpaceChar
                    clsString.Append Chr(NoBreakSpace)
                Case Else
                    clsString.Append "&#" & currChar & ";"
            End Select

            ' The last position is now this plus one
            lngLast = Tell + 1

        End If
        
        ' Store this character
        prevChar = currChar
    
    Next
    
    ' See if we need to add a string
    If lngLast < Len(sText) Then
        ' Add the string
        clsString.Append Mid(sText, lngLast)
    End If
    
    ' Clear up
    ZeroMemory ByVal VarPtrArray(intChar), 4
    
End Sub

Public Function ValidPath(Path As String) As String

    ValidPath = Path & IIf(Right(Path, 1) = "\", "", "\")

End Function

Public Sub RunScript(sCode As String)

    Dim Script As Object

    ' Create script-engine
    Set Script = CreateObject("MSScriptControl.ScriptControl")
    
    ' Set language and add objects
    Script.Language = "VBScript"
    Script.AddObject "Convert", Me

    ' Initialize the code-begin class
    Set CodeBegin = New StringBuilder

    ' Set settings
    Script.AddCode sCode

End Sub

Public Sub InitializeCode(sFileName As String)

    ' Run script
    RunScript ReadFile(ValidPath(App.Path) & "Scripts/" & sFileName)

End Sub

Private Sub Class_Initialize()

    ' Load all resources
    RunScript ReadFile(ValidPath(App.Path) & "Resources\Keywords.vbs")
    
    ' Default value
    TabReplacement = String(4, Chr(NoBreakSpace))

End Sub

Public Property Get VBClipboard() As Object

    ' Return the VB-clipboard
    Set VBClipboard = Clipboard

End Property
